"""Module containing a simple logging implementation.

Logged messages are by default wrapped to width of the current terminal window,
multi-line messages are pretty-printed.

Output is colorized according to "severity":

- info (no color)
- errors (red)
- warnings (yellow)
- success (green)
"""

import shutil
import sys
import textwrap

from termcolor import colored

NOWRAP = False
"""Global variable that controls whether log messages are wrapped to the width
of the current terminal window or not. Setting this to `True` can be useful for
debugging or testing purposes, since it makes the messages printed to stderr
deterministic and independent of the current environment."""


def _eprint(message: str):
    # T201: this print is used to implement custom logging
    print(message, file=sys.stderr)  # noqa: T201


def _wrap(message: str, prefix: str, width: int | None = None) -> list[str]:
    if NOWRAP:
        return [f"{prefix} {message}"]

    if not width:
        width = shutil.get_terminal_size().columns

    return textwrap.wrap(message, 80, initial_indent=f"{prefix} ", subsequent_indent=" " * (len(prefix) + 1))


def success(message: str):
    """Log a message for an action that finished successfully."""
    _eprint(colored("\n".join(_wrap(message, "•")), "green"))


def info(message: str):
    """Log a purely informational message."""
    _eprint("\n".join(_wrap(message, "•")))


def warn(message: str):
    """Log a warning message.

    This is associated with potential problems that are however non-fatal.
    """
    _eprint(colored("\n".join(_wrap(message, "WARNING:")), "yellow"))


def error(message: str):
    """Log an error message.

    This is usually associated with hard errors that cause the program to exit.
    """
    _eprint(colored("\n".join(_wrap(message, "ERROR:")), "red", attrs=["dark"]))
