"""Module containing helper functionality for parsing / processing crate metadata."""

from cargo2rpm.metadata import Metadata, Package
from cargo2rpm.semver import Comparator, VersionReq
from cargo2rpm.semver import Op as Operator

from rust2rpm import log


class WorkspaceError(Exception):
    """Raised if the 'main' package of a workspace cannot be determined."""


def get_required_features_for_binaries(package: Package) -> set[str]:
    """Query crate metadata for the set of features that is required by binary targets.

    Arguments:
        package: Crate metadata.

    Returns:
        Union of the required features of all `bin` and `cdylib` targets.

    """
    required_features = set()

    for target in package.targets:
        if "bin" in target.kind and "bin" in target.crate_types and (reqs := target.required_features):
            required_features.update(reqs)
        if "cdylib" in target.kind and "cdylib" in target.crate_types and (reqs := target.required_features):
            required_features.update(reqs)

    return required_features


def guess_main_package(metadata: Metadata, hint: str | None, guess: str | None) -> Package:
    """Guess the "main" member of a cargo workspace.

    This function uses some heuristics to try to determine the "main"
    member of a cargo workspace:

    - If there is a workspace member with a name that matches the supplied
        hint, that workspace member is returned.
    - If there is exactly one workspace member that contains `cdylib` targets,
        that workspace member is returned.
    - If there is exactly one workspace member that contains `bin` targets,
        that workspace member is returned.

    Arguments:
        metadata: Metadata for the whole cargo workspace.
        hint: Optional hint pointing at the "main" workspace member.
        guess: Optional guess for the project name based on the name of the
            parent directory.

    Returns:
        Metadata for the "main" workspace member.

    Raises:
        `WorkspaceError` is raised in cases where the "main" package cannot be
            determined heuristically or the supplied hint was invalid.

    """
    if hint:
        for package in metadata.packages:
            if package.name == hint:
                return package

        msg = "The supplied hint for the name of the 'main' crate of the did not match any workspace member."
        raise WorkspaceError(msg)

    with_bin = []
    with_cdylib = []

    for package in metadata.packages:
        for target in package.targets:
            if "bin" in target.kind and "bin" in target.crate_types:
                with_bin.append(package)
            if "cdylib" in target.kind and "cdylib" in target.crate_types:
                with_cdylib.append(package)

    if len(with_cdylib) == 1:
        return with_cdylib[0]

    if len(with_bin) == 1:
        return with_bin[0]

    if guess:
        for package in metadata.packages:
            if package.name == guess:
                return package

    msg = (
        "Heuristic for determining the 'main' crate of the workspace failed. "
        "Please supply the 'pkgid' argument to select the 'main' workspace member."
    )
    raise WorkspaceError(msg)


def package_uses_rust_1_60_feature_syntax(features: dict[str, list[str]]) -> bool:
    """Determine whether the specified feature set contains syntax only supported in Rust 1.60+.

    Arguments:
        features: Mapping from feature names to feature dependencies.

    Returns:
        `True` if the new feature syntax is used, and `False` otherwise.

    """
    for name, deps in features.items():
        for dep in deps:
            if "?/" in dep:
                return True

            if dep.startswith("dep:"):
                if len(deps) != 1:
                    return True
                if dep.removeprefix("dep:") != name:
                    return True

    return False


def warn_if_package_uses_restrictive_dependencies(package: Package):
    """Log a warning if crate metadata contains restrictive version ranges for dependencies.

    Dependencies with versions ranges that are more restrictive than
    "SemVer-compatible" cause problems with packaging, and the reasons for their
    presence are in almost all cases not valid reasons for downstream packaging
    (i.e. MSRV concerns). This logs a warning if the given crate metadata
    contains any such dependencies.

    Arguments:
        package: Metadata for a single crate.

    """
    for dependency in package.dependencies:
        name = dependency.rename or dependency.name
        req = VersionReq.parse(dependency.req)
        for comp in req.comparators:
            if _is_strict_dep(comp):
                log.warn(f"Dependency on {name!r} stricter than necessary: {comp}")


def _is_strict_dep(comp: Comparator) -> bool:
    # dependencies like "~1.0" are stricter than SemVer
    if comp.op == Operator.TILDE and comp.major > 0 and comp.minor is not None:
        return True

    # dependencies like "1.0.*" are stricter than SemVer
    if comp.op == Operator.WILDCARD and comp.major > 0 and comp.minor is not None:
        return True

    # dependencies like "=1.2" are stricter than SemVer
    if comp.op == Operator.EXACT and comp.major > 0 and comp.minor is not None and comp.patch is None:  # noqa: SIM103
        return True

    return False
