"""Module containing functionality for handling tarballs containing vendored sources."""

import shutil
import subprocess
from pathlib import Path

from rust2rpm import log
from rust2rpm.cli import VendorMode


def generate_vendor_tarball(
    toml_path: Path,
    project_name: str,
    project_version: str,
    dest_dir: Path,
    mode: VendorMode,
) -> str | None:  # pragma nocover: requires internet access
    """Return a default name for a tarball containing vendored sources.

    There are three different modes of operation depending on the `VendorMode`:

    - `OFF`: do nothing and return `None`
    - `MANUAL`: do nothing and return a file name in the default format
    - `AUTO`: create a tarball containing vendored sources, with default settings,
        move it to `dest_dir`, and return its file name

    Arguments:
        toml_path: Path to the Cargo.toml file of the project for which
            a tarball with vendored sources will be generated.
        project_name: Name of the project used in the default filename.
        project_version: Version of the project used in the default filename.
        dest_dir: Output directory for the generated vendor tarball.
        mode: Mode of operation.

    Returns:
        File name of the generated tarball or `None` if no tarball is used.
        Returns `None` in `VendorMode` is `OFF`, and a file name in default
        format for other modes.

    """
    if mode == VendorMode.OFF:
        return None

    tarball_name = f"{project_name}-{project_version}-vendor.tar.xz"
    tarball_dest = dest_dir / tarball_name

    if mode == VendorMode.MANUAL:
        log.info(f"Assuming manually created vendor tarball named {tarball_name}")
        return tarball_name

    # mode == VendorMode.AUTO:

    if tarball_dest.is_file():
        log.info(f"Vendor tarball already exists: {tarball_name}")
        log.info("To force re-generation of the vendor tarball, remove this file.")
        return tarball_name

    log.info("Generating vendor tarball ...")

    # S603: these commands to not take inputs that would need to be validated
    # S607: allowing use of "cargo" from $PATH is intentional

    subprocess.run(  # noqa: S603
        ["cargo", "vendor", "--versioned-dirs", "--manifest-path", toml_path, "vendor/"],  # noqa: S607
        capture_output=True,
        check=True,
    )

    log.info("Compressing vendor tarball ...")

    subprocess.run(  # noqa: S603
        ["/usr/bin/tar", "-cJvf", tarball_name, "vendor/"],
        capture_output=True,
        check=True,
    )

    shutil.rmtree("vendor/")

    if dest_dir != Path.cwd():
        Path(tarball_name).rename(tarball_dest)

    return tarball_name
